/* Beholder RMON ethernet network monitor,Copyright (C) 1993 DNPAP group */
/* See file COPYING 'GNU General Public Licence' for copyright details   */
/************************************************************************
 ** MODULE INFORMATION*
 **********************
 **     FILE     NAME:       api.c
 **     SYSTEM   NAME:       mib
 **     ORIGINAL AUTHOR(S):  Jan van Oorschot
 **     VERSION  NUMBER:     1.00
 **     CREATION DATE:       
 **
 ** DESCRIPTION: This file contains the API calls supported my the
 **              MIB library.
 *************************************************************************
 ** CHANGES INFORMATION **
 *************************
 ** REVISION:    $Revision: 1.1.1.1 $
 ** WORKFILE:    $Workfile$
 ** LOGINFO:     $Log: mib_api.c,v $
 ** LOGINFO:     Revision 1.1.1.1  1998/07/14 06:33:27  bambi
 ** LOGINFO:
 *************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <ctype.h>

#include "common.h"
#include "snmp.h"
#include "mib.h"




/**************************************************************
 ** NAME:        MibFullMib
 ** SYNOPSIS:    iRet = MibFullMib();
 **              
 ** DESCRIPTION: Reads the file $MIBHOME/mibs and interprets each
 **     line as the name of a MOSI mib file in $MIBHOME
 **     Puts these
 **     files in the tree.
 **     MIBHOME should be something like 
 **
 **         /usr/local/lib/mibhome/
 **
 **      this functions will use the following logic to locate
 **      $MIBHOME:
 **
 **          . search the .ini files
 **          . search environment variable MIBHOME
 **          . use value of macro $(MIBHOME)
 **
 ** RETURNS:     0 -->   no error
 **              else    error code
 **************************************************************/
int MibFullMib()
{
	FILE *fp1, *fp2;
	char *mibhome = NULL;
	char fname[265], pname[256];
	int len, iret;

	/* use the environment variable */
	if (mibhome == NULL)
		mibhome = getenv("MIBHOME");

	/* use the macro */
	if (mibhome == NULL)
		mibhome = (char *) MIBHOME;

	sprintf(pname,"%s/mibs/mibs",mibhome);

	fp1 = fopen(pname, "r");
	if (fp1 == NULL)
	{
		sprintf(snmpError, "Could not find MIB catalog '%s',\n",pname);
		return -1;
	}
	while (!feof(fp1))
	{
		sprintf(pname,"%s/mibs",mibhome);
		fgets(fname, 256, fp1);
		len = strlen(fname);
		if (len > 1)
		{
			if (fname[len - 1] == '\n')
				fname[len - 1] = '\0';
			strcat(pname, "/");
			strcat(pname, fname);
			fp2 = fopen(pname, "r");

			if (fp2 != NULL)
			{
				iret = MibReadMib(fp2);
			}
			else
			{
				sprintf(snmpError,
					"could not read mib file <%s>\n",
					pname);
				return(-1);
			}
		}
	}
	return 0;
}

/**************************************************************
 ** NAME:        MibReadMib
 ** SYNOPSIS:    iRet = MibReadMib(fp);
 **     FILE *fp;
 **              
 ** DESCRIPTION: Reads a MIB definition from file-stream <fp>
 **     up to the EOF.  The MIB read from this file
 **     is stored in the global MIB tree.
 ** RETURNS:     0 -->   no error
 **              else    error code
 **************************************************************/
int MibReadMib(fp)
	FILE * fp;
{
	char line[256];
	MIBNODE *node;

	while (!feof(fp))
	{
		fgets(line, 256, fp);
		node = MibScanLine(line);
	}
	return 0;
}



/**************************************************************
 ** NAME:   MibName2ID        
 ** SYNOPSIS:    int = MibName2ID(id,name);
 **     LONG    *id;
 **     char    *name;
 **              
 ** DESCRIPTION: Translates the <name> of a MIB variable 
 **     in the <id> of that variable.
 ** RETURNS:     <0     error
 **     else    length of id
 **************************************************************/
int MibName2ID(id, name)
	long 	*id; 
	char 	*name;
{
	extern MIBNODE *mibRoot;
	MIBNODE *curNode, *curChild;
	int	index, len;
	char	*fullName, nameDup[255], *tok;

	fullName = (char *)MibGetFullName(name);
	strcpy(nameDup, fullName);

	tok = strtok(nameDup,".");
	curNode = mibRoot;

	if (strcasecmp(tok,"iso") != 0 && atoi(tok) != 1)
	{
		sprintf(snmpError,"Variable does not start with the ISO root");
		return(-1);
	}
	len = 0;
	id[len++] = curNode->index;
	tok = strtok(NULL,".");
	while(tok)
	{
		if (isdigit(*tok))
			index = atoi(tok);
		else
			index = 0;
		curChild = curNode->child1;
		while(curChild)
		{
			if (index)
			{
				if (curChild->index == index)
					break;
			}
			else
			{
				if (strcasecmp(curChild->name,tok) == 0)
					break;
			}
			curChild = curChild->brother;
		}
		if (!curChild)
		{
			break;
		}
		id[len++] = curChild->index;
		curNode = curChild;
		tok = strtok(NULL,".");
	}

	/*
	** Is there a trailing numeric token?
	*/
	while(tok)
	{
		id[len++] = atoi(tok);
		tok = strtok(NULL,".");
	}
	return(len);
}



/**************************************************************
 ** NAME:        MibID2Name
 ** SYNOPSIS:    cPtr = MibID2Name(id,nr,nrmatched);
 **     LONG    *id;
 **     int     nr;
 **     int     *nrmatched
 **              
 ** DESCRIPTION: Translates the given <id> with length <nr>
 **     to the corresponding logical MIB name. Returns
 **     a pointer to that name and fills <nrmatched>
 **     with the number of matched sub-identifiers
 ** RETURNS:     0 -->   no error
 **              else    error code
 **************************************************************/

char *MibID2Name(id, nr, nrmatched, buf)
	long	*id;
	int	nr, *nrmatched;
	char	*buf;
{
	extern MIBNODE *mibRoot;
        MIBNODE *node;
        int cntr;
	char	tmpBuf[10];

        node = mibRoot;

	strcpy(buf,node->name);
	*nrmatched = 0;
        for (cntr = 1; cntr < nr; cntr++)
        {
                node = MibFindChild(node, id[cntr]);
		if (node)
		{
			strcat(buf,".");
			strcat(buf,node->name);
		}
		else
		{
			while (cntr < nr)
			{
				strcat(buf,".");
				sprintf(tmpBuf,"%ld",id[cntr++]);
				strcat(buf,tmpBuf);
			}
			break;
		}
        }
	*nrmatched = cntr - 1;
        return buf;
}



/**************************************************************
 ** NAME:        MibIDType
 ** SYNOPSIS:    iRet =  MibIDType(id,nr);
 **              LONG   *id;
 **              int        iRet,nr;
 ** DESCRIPTION: Returns the type if the MIB variable identified
 **             by <id> of lenght <nr>
 ** RETURNS:     <0 error
 **              else    mode
 **************************************************************/
int MibIDType(id, nr)
	long *id; 
	int nr;
{
	MIBNODE *node;

	node = MibFindID(id, nr);
	if (node == NULL)
		return -1;
	return node->type;
}





/**************************************************************
 ** NAME:        MibIDMode
 ** SYNOPSIS:    iret = MibIDMode(id,nr);
 **              LONG *id;
 **              int nr,iRet;
 ** DESCRIPTION: Returns the mode of the MIB variable identified
 **     by <id> of length <nr>
 ** RETURNS:     <0 error
 **     mode
 **************************************************************/
int MibIDMode(id, nr)
	long *id; 
	int nr;
{
	MIBNODE *node;

	node = MibFindID(id, nr);
	if (node == NULL)
		return -1;
	return node->mode;
}





/**************************************************************
 ** NAME:        MibIDStatus
 ** SYNOPSIS:    iRet = MibIDStatus(id,nr);
 **             LONG    *id;
 **             int nr,iRet;
 ** DESCRIPTION: Returns the status of the MIB varaible 
 **     identified by <id> of length <nr>.
 ** RETURNS:     <0 error
 **              status
 ** RETURNS:     void
 **************************************************************/
int MibIDStatus(id, nr)
	long *id; 
	int nr;
{
	MIBNODE *node;

	node = MibFindID(id, nr);
	if (node == NULL)
		return -1;
	return node->status;
}



/**************************************************************
 ** NAME:        MibIDInfo
 ** SYNOPSIS:    iRet = MibIDInfo(id,nr,res);
 **             LONG    *id;
 **             int nr,iRet;
 **         char    *res;
 ** DESCRIPTION: stores information about the given
 **     MIB <id> in character buffer <res>
 ** RETURNS:     <0 error
 **              status
 ** RETURNS:     void
 **************************************************************/
int MibIDInfo(id, nr, res)
	long 	*id; 
	int 	nr; 
	char 	*res;
{
	char *type, *mode, *status;
	MIBNODE *node;

	node = MibFindID(id, nr);
	if (node == NULL)
	{
		sprintf(snmpError,"unknown MIB OID");
		return -1;
	}
	if ((type = MibEncodeType(node->type)) == NULL)
		type = "";
	if ((mode = MibEncodeMode(node->mode)) == NULL)
		mode = "";
	if ((status = MibEncodeStatus(node->status)) == NULL)
		status = "";
	strcpy(res, node->name);
	strcat(res, " ");
	strcat(res, type);
	strcat(res, " ");
	strcat(res, mode);
	strcat(res, " ");
	strcat(res, status);
	return 0;
}




/**************************************************************
 ** NAME:        MibNameType
 ** SYNOPSIS:    iRet = MibNameType(name)
 **     char    *name;
 **
 ** DESCRIPTION: Returns the type of the MIB varaible 
 **     identified by <name>
 ** RETURNS:     <0 error
 **              type
 ** RETURNS:     void
 **************************************************************/
int MibNameType(name)
	char 	*name;
{
	MIBNODE *node;

	node = MibFindNode(name);
	if (node == NULL)
	{
		sprintf(snmpError,"unknown MIB variable '%s'",name);
		return -1;
	}
	return node->type;
}




/**************************************************************
 ** NAME:        MibNameMode
 ** SYNOPSIS:    iRet = MibNameMode(name)
 **     char    *name;
 **
 ** DESCRIPTION: Returns the mode of the MIB varaible 
 **     identified by <name>
 ** RETURNS:     <0 error
 **              mode
 ** RETURNS:     void
 **************************************************************/
int MibNameMode(name)
	char *name;
{
	MIBNODE *node;

	node = MibFindNode(name);
	if (node == NULL)
	{
		sprintf(snmpError,"unknown MIB variable '%s'",name);
		return -1;
	}
	return node->mode;
}



/**************************************************************
 ** NAME:        MibNameStatus
 ** SYNOPSIS:    iRet = MibNameStatus(name)
 **     char    *name;
 **
 ** DESCRIPTION: Returns the status of the MIB varaible 
 **     identified by <name>
 ** RETURNS:     <0 error
 **              status
 ** RETURNS:     void
 **************************************************************/
int MibNameStatus(name)
	char 	*name;
{
	MIBNODE *node;

	node = MibFindNode(name);
	if (node == NULL)
	{
		sprintf(snmpError,"unknown MIB variable '%s'",name);
		return -1;
	}
	return node->mode;
}




/**************************************************************
 ** NAME:        MibGetFirstChild
 ** SYNOPSIS:    char * MibGetFirstChild(name);
 **     char    *name;
 ** DESCRIPTION: return pointer to name of first 
 **     child of <name>
 ** RETURNS:     NULL   no first child
 **              ptr to name first child
 ** RETURNS:     void
 **************************************************************/
char *MibGetFirstChild(name)
	char 	*name;
{
	MIBNODE *node, *fchild;

	node = MibFindNode(name);
	if (node == NULL)
	{
		sprintf(snmpError,"unknown MIB variable '%s'",name);
		return NULL;
	}
	fchild = node->child1;
	if (fchild == NULL)
		return NULL;
	return fchild->name;
}



/**************************************************************
 ** NAME:        MibGetNextChild
 ** SYNOPSIS:    char * MibGetNextChild(name);
 **     char    *name;
 ** DESCRIPTION: return pointer to name of next 
 **     child from child <name>
 ** RETURNS:     NULL   no next child
 **              ptr to name next child
 ** RETURNS:     void
 **************************************************************/
char *MibGetNextChild(name)
	char 	*name;
{
	MIBNODE *node, *fchild;

	node = MibFindNode(name);
	if (node == NULL)
	{
		sprintf(snmpError,"unknown MIB variable '%s'",name);
		return NULL;
	}
	fchild = node->brother;
	if (fchild == NULL)
		return NULL;
	return fchild->name;
}
