/* Beholder RMON ethernet network monitor,Copyright (C) 1993 DNPAP group */
/* See file COPYING 'GNU General Public Licence' for copyright details   */
/************************************************************************
 ** MODULE INFORMATION*
 **********************
 **     FILE     NAME:       mib.c
 **     SYSTEM   NAME:       Sage
 **     ORIGINAL AUTHOR(S):  Jan van Oorschot
 **     VERSION  NUMBER:     1.00
 **     CREATION DATE:       
 **
 ** DESCRIPTION: Main MIB handling routine.
 **              
 *************************************************************************
 ** CHANGES INFORMATION **
 *************************
 ** REVISION:    $Revision: 1.1.1.1 $
 ** WORKFILE:    $Workfile$
 ** LOGINFO:     $Log: mibdf.c,v $
 ** LOGINFO:     Revision 1.1.1.1  1998/07/14 06:33:27  bambi
 ** LOGINFO:
 *************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "common.h"
#include "snmp.h"
#include "mib.h"


MIBNODE *mibRoot = NULL;
static MIBHASH *mibHash = NULL;


struct mib_prefix {
        char    *name,
                *prefix;
};

static struct mib_prefix prefixes[] = {
	{ "experimental.", "iso.org.dod.internet.experimental"},
	{ "enterprises.", "iso.org.dod.internet.private.enterprises"},
	{ "private.", "iso.org.dod.internet.private"},
	{ "", "iso.org.dod.internet.mgmt.mib-2" }, /* must be last! */
	{ NULL, NULL }
};


int MibInitialise()
{
	int i;

	/* initialise the tree */
	mibRoot = (MIBNODE *)malloc(sizeof(MIBNODE));
	mibRoot->name = "iso";
	mibRoot->index = 1;
	mibRoot->type = 0;
	mibRoot->mode = 0;
	mibRoot->status = 0;
	mibRoot->father = mibRoot->brother = mibRoot->child1 = NULL;

	/* initialise the hash table */
        mibHash = (MIBHASH *)malloc(MIBHASHSIZE * sizeof(MIBHASH));
        if (mibHash == NULL)
        {
                sprintf(snmpError,"out of memory");
                return(-1);
        }
        for (i = 0; i < MIBHASHSIZE; i++)
        {
                mibHash[i].next = NULL;
                mibHash[i].contents = NULL;
        }

        /* add the tree root to the hash table */
        mibHash[MibNameHash(mibRoot->name)].contents = mibRoot;
	return(0);
}



int MibNameHash(name)
	char 	*name;
{
        int j = 0;

        while (*name)
                j = (j * 3 + *name++) % MIBHASHSIZE;
        return j;
}



void MibAddHash(name, ptr)
	char 	*name; 
	MIBNODE *ptr;
{
        MIBHASH *mptr, *nptr;

        /* mptr points to the first of the list */
        mptr = &(mibHash[MibNameHash(name)]);
        if (mptr->contents == NULL)
        {
                mptr->contents = ptr;
        }
        else
        {
                nptr = (MIBHASH *)malloc(sizeof(MIBHASH));
                nptr->next = mptr->next;
                nptr->contents = ptr;
                mptr->next = nptr;
        }
}


MIBNODE *MibResolveHash(name)
	char 	*name;
{
        MIBHASH *mptr;

        /* simple implementation, liniear search in hash entry */
        mptr = &(mibHash[MibNameHash(name)]);
        if (mptr->contents == NULL)
                return NULL;
        while (strcmp(mptr->contents->name, name) != 0)
                if ((mptr = mptr->next) == NULL)
                        return NULL;
        return mptr->contents;
}



MIBNODE *MibFindChild(fPtr, index)
	MIBNODE *fPtr; 
	long 	index;
{
	MIBNODE *cPtr;

	cPtr = fPtr->child1;

	while (cPtr != NULL)
	{
		if (cPtr->index == index)
			return cPtr;
		cPtr = cPtr->brother;
	}
	return NULL;
}



MIBNODE *MibLookup(name)
	char 	*name;
{
	char 	*tok, 
		*tmpTok,
		nameDup[255];
	MIBNODE *curNode, *curChild;
	int 	index;

	strcpy(nameDup, name);
	tok = strtok(nameDup,".");
	curNode = mibRoot;

	/*
	** Special case.  The first token has to be "iso" otherwise we
	** can't start at the root.  The prefix handling will ensire
	** this unless someone sends us an absolute path (i.e. prefixed
	** with a '.' that doesn't start with "iso"
	*/
	if (strcasecmp(tok,"iso") != 0)
	{
		return(NULL);
	}
	tok = strtok(NULL,".");
	while(tok)
	{
		if (isdigit(*tok))
			index = atoi(tok);
		else
			index = 0;
		curChild = curNode->child1;
		while(curChild)
		{
			if (index)
			{
				if (curChild->index == index)
					break;
			}
			else
			{
				if (strcasecmp(curChild->name,tok) == 0)
					break;
			}
			curChild = curChild->brother;
		}
		if (!curChild)
		{
			tmpTok = strtok(NULL,".");
			if (tmpTok)
				return(NULL);
			if (! isdigit(*tok))
				return(NULL);
			break;
		}
		curNode = curChild;
		tok = strtok(NULL,".");
	}
	return(curNode);
}



char *MibGetFullName(name)
	char	*name;
{
	static 	char fullName[255];
	struct	mib_prefix *curPrefix;

	/*
	** Handle absolute variables
	*/
	if ( *name == '.')
	{
		return(name+1);
	}

	/*
	** Now try prefixing
	*/
	curPrefix = prefixes;
	while(curPrefix)
	{
		if (strncasecmp(name, curPrefix->name,
			strlen(curPrefix->name)) == 0)
		{
			sprintf(fullName,"%s.%s",curPrefix->prefix,
				name + strlen(curPrefix->name));
			return(fullName);
		}
		curPrefix++;
	}
	return(name);
}




MIBNODE *MibFindNode(name)
	char 	*name;
{
	if (!name)
	{
		return(NULL);
	}
	return(MibLookup(MibGetFullName(name)));
}




void MibStripPrefix(var)
	char	*var;
{
	struct	mib_prefix *curPrefix;
	char	*cp1, *cp2;

	curPrefix = prefixes;
	while(curPrefix)
	{
		if (strncasecmp(var, curPrefix->prefix,
			strlen(curPrefix->prefix)) == 0)
		{
			strcpy(var, curPrefix->name);
			cp1 = var + strlen(curPrefix->name);
			cp2 = var + strlen(curPrefix->prefix) + 1;
			while(*cp2)
			{
				*cp1++ = *cp2++;
			}
			*cp1 = 0;
			return;
		}
		curPrefix++;
	}
}




int MibAddNode(node, fnode)
	MIBNODE * node,*fnode;
{
	MIBNODE *bnode;
	int bfound;

	/* node should contain a valid name and index */
	if (fnode == NULL)
		return -1;
	/* check if this is the first child */
	if (fnode->child1 == NULL)
	{
		fnode->child1 = node;
		node->father = fnode;
		node->brother = NULL;
		node->child1 = NULL;
                MibAddHash(node->name, node);
		return 0;
	}

	/* check if this node should be the first child of father */
	if (fnode->child1->index == node->index)
		return -2;
	if (fnode->child1->index > node->index)
	{
		node->brother = fnode->child1;
		fnode->child1 = node;
		node->father = fnode;
		node->child1 = NULL;
                MibAddHash(node->name, node);
		return 0;
	}

	/* insert node in brother-list */
	bnode = fnode->child1;
	bfound = 0;
	while (!bfound)
	{
		if (bnode->brother == NULL)
			bfound = 1;
		else if (bnode->brother->index >= node->index)
			if (bnode->brother->index == node->index)
				return -1;
			else
				bfound = 1;
		else
			bnode = bnode->brother;
	}
	/* insert in brother-list */
	node->brother = bnode->brother;
	node->father = fnode;
	node->child1 = NULL;
	bnode->brother = node;
	MibAddHash(node->name, node);

	return 0;
}




int MibFillID(node, id)
	MIBNODE *node; 
	long 	*id;
{
	int indexnr;

	if (node->father == NULL)
	{
		/* at top , fill id and return sequence number */
		*id = node->index;
		return 1;
	}
	else
	{
		/* recursive call, then fill */
		indexnr = MibFillID(node->father, id);
		*(id + indexnr) = node->index;
		return ++indexnr;
	}
}




MIBNODE *MibFindID(id, nr)
	long 	*id; 
	int 	nr;
{
	MIBNODE *node;
	int cntr;

	node = mibRoot;

	for (cntr = 1; cntr < nr; cntr++)
	{
		if (node == NULL)
			return NULL;
		node = MibFindChild(node, id[cntr]);
	}

	return node;
}




MIBNODE *MibFindID2(id, nr, nrmatched)
	long 	*id; 
	int 	nr, *nrmatched;
{
	MIBNODE *node, *lastnode;
	int cntr;

	node = lastnode = mibRoot;

	for (cntr = 1; cntr < nr; cntr++)
	{
		node = MibFindChild(lastnode, id[cntr]);
		if (node == NULL)
		{
			*nrmatched = cntr;
			return lastnode;
		}
		else
		{
			lastnode = node;
		}
	}
	*nrmatched = cntr;
	return node;
}




void MibDumpSubTree(first, path)
	MIBNODE *first;
	char	*path;
{
	MIBNODE *cptr;
	char	full[255];

	printf("%s.%s:%ld\n", path, first->name, first->index);
	cptr = first->child1;
	while (cptr != NULL)
	{
		sprintf(full,"%s.%s",path,first->name);
		MibDumpSubTree(cptr,full);
		cptr = cptr->brother;
	}
}

void MibDumpTree()
{
	MibDumpSubTree(mibRoot,"");
}
