/* Beholder RMON ethernet network monitor,Copyright (C) 1993 DNPAP group */
/* See file COPYING 'GNU General Public Licence' for copyright details   */


#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <ctype.h>

#include "common.h"
#include "snmp.h"
#include "mib.h"


int snmpAddVariable(name, object)
	char 	*name; 
	SNMP_OBJECT *object;
{
	int i = 0, r = 0, n = 0;

	while (name[i] == ' ' || name[i] == '\t')
		i++;
	n = MibName2ID(object->Id, name);
	if (n < 0)
		return -1;
	object->IdLen = n;
	i += r;
	object->Type = SNMP_NULL;
	object->Syntax.LngUns = 0L;
	return i;
}





int snmpProcessResult(obj, numObj)
	SNMP_OBJECT 	*obj; 
	int 		numObj;
{
	int n = 0, count = 0;
	SNMP_OBJECT 	*curObj; 

	curObj = obj;
	while(count < numObj)
	{
		if (MibID2Name(curObj->Id, (int) (curObj->IdLen),&n,
			curObj->textID)==NULL)
		{
			sprintf(snmpError,
				"variable %d in list is not implemented",
				count);
			return(-1);
		}
		MibStripPrefix(curObj->textID);
		sprintValue(curObj, curObj->textVal);
		count ++;
		curObj++;
	}
	return(0);
}




int sprintValue(object, buf)
	SNMP_OBJECT *object; 
	char 	*buf; 
{
	int i = 0, r = 0, u;
        int nr;
	ULONG lu;
	USHORT n = 0;
	BYTE *ptr;
	char	tmpBuf[1024];

	switch (object->Type)
	{
	case SNMP_NULL:
		sprintf(buf + i, "NULL");
		i += 4;
		break;
	case SNMP_INTEGER:
		sprintf(buf + i, "%ld%n", object->Syntax.LngInt, &r);
		i += r;
		break;
	case SNMP_OCTETSTR:
		if (IsVisible(object->Syntax.BufChr, (int) (object->SyntaxLen)))
		{
			object->Syntax.BufChr[object->SyntaxLen] = '\0';
			/* copy string, replace \n and double \ */
			ptr = object->Syntax.BufChr;
			while (*ptr != '\0')
			{
				if (*ptr == '\n' || *ptr == '\r')
				{
					buf[i++] = '\\';
					buf[i++] = 'n';
				}
				else
				{
					if (*ptr == '\\')
					{
						buf[i++] = '\\';
						buf[i++] = '\\';
					}
					else
					{
						buf[i++] = *ptr;
					}
				}
				ptr++;
			}
			buf[i] = 0;
			i++;
		}
		else
		{
			u = (BYTE) object->Syntax.BufChr[0];
			if (sprintf(buf + i, "%02x%n", u, &r) == 0)
				return (-1);
			i += r;
			for (n = 1; n < object->SyntaxLen; n++)
			{
				u = (BYTE) object->Syntax.BufChr[n];
				if (sprintf(buf + i, ":%02x%n", u, &r) == 0)
					return (-1);
				i += r;
			}
		}
		break;
	case SNMP_OBJECTID:
                /* resolve the name as far as possible */
                if((ptr = (BYTE *)MibID2Name(object->Syntax.BufInt,
			(int) object->SyntaxLen, &nr,tmpBuf)) == NULL)
                {
                     nr = 0;
                }
                else
                {
                     sprintf(buf+i,"%s",ptr);
                     i+=strlen(ptr);
                }
                if(nr < object->SyntaxLen)
                {
                     for(n=nr;n<object->SyntaxLen;n++)
                     {
                          lu = object->Syntax.BufInt[n];
                          sprintf(buf+i,"%lu.%n",lu,&r);
                          i+=r;
                     }
                }
		break;
	case SNMP_IPADDR:
		u = (BYTE) object->Syntax.BufChr[0];
		if (sprintf(buf + i, "%u%n", u, &r) == 0)
			return (-1);
		i += r;
		for (n = 1; n < 4; n++)
		{
			u = (BYTE) object->Syntax.BufChr[n];
			if (sprintf(buf + i, ".%u%n", u, &r) == 0)
				return (-1);
			i += r;
		}
		break;
	case SNMP_COUNTER:
	case SNMP_GAUGE:
		sprintf(buf + i, "%lu%n", object->Syntax.LngUns, &r);
		i += r;
		break;
	case SNMP_TIMETICKS:
		/* a time format would be nice, but other utils may want to use this
		 * value as a simple number and would then be worse off */
		sprintf(buf + i, "%lu%n", object->Syntax.LngUns, &r);
		i += r;
		break;
	case SNMP_OPAQUE:
		break;
	}
	return i;
}





int IsVisible(s, n)
	BYTE * s; 
	int n;
{
	BYTE c;

	while (n-- > 0)
	{
		c = *s++;
		if (c > 126 || (c < 32 && c != 9 && c != 10 && c != 13))
			return 0;
	}
	return 1;
}


int snmpCheckVariable(var)
	char 	*var; 
{
	SNMP_OBJECT object;
	int i = 0, n = 0;

	while (var[i] == ' ' || var[i] == '\t')
		i++;
	n = MibName2ID(object.Id, var);
	if (n < 0)
		return -1;
	return(0);
}
