
#include        <stdio.h>
#include 	<stdlib.h>
#include	<unistd.h>
#include        <string.h>

#include 	"common.h"
#include        "snmp.h"



int snmpInit()
{
	if (MibInitialise() < 0)
		return(-1);
	if (MibFullMib() < 0)
		return(-1);
	return(0);
}

	

snmp *snmpOpen(host, community, timeout, retries)
	char 	*host,
		*community;
	unsigned timeout;
	int	retries;
{
	snmp	*new;

	if (host == NULL)
	{
		sprintf(snmpError,"missing host name");
		return(NULL);
	}
	if (community == NULL)
	{
		sprintf(snmpError,"missing community string");
		return(NULL);
	}


	new = (snmp *)malloc(sizeof(snmp));
	if (new == NULL)
	{
		sprintf(snmpError,"out of memory");
		return NULL;
	}
	strcpy(new->community, community);
	if (timeout == 0)
		new->timeout = SNMP_TIMEOUT;
	else
		new->timeout = timeout;
	if (retries == 0)
		new->retries = SNMP_RETRIES;
	else
		new->retries = retries;
	new->sock = UdpOpen(host);
	if (new->sock < 0)
	{
		free(new);
		return(NULL);
	}
	return(new);
}



void snmpClose(session)
	snmp *session;
{
	close(session->sock);
	free(session);
}






static BOOLEAN snmpRequest(session, type, list, listLen, status, index)
	snmp 	*session; 
	WORD 	type; 
	SNMP_OBJECT * list; 
	unsigned listLen; 
	WORD * status, * index; 
{
	static ULONG id = 0;
	SNMP_PDU pdu;
	BYTE *rqs;
	unsigned rqsLen;
	BYTE *rsp;
	unsigned rspLen;
	BYTE commRsp[SNMP_SIZE_COMM];
	unsigned commLenRsp;
	unsigned listLenRsp;
	BOOLEAN success;
	BYTE buffer[SNMP_SIZE_COMM];
	int try = 0;

	success = FALSE;

	rqs = buffer;
	rqsLen = BUF_LEN;

	pdu.Type = type;
	pdu.Request.Id = ++id;
	pdu.Request.ErrorStatus = SNMP_NOERROR;
	pdu.Request.ErrorIndex = 0;

	if(SnmpEnc(&rqs, &rqsLen, &pdu, session->community,
		strlen(session->community), list, listLen) == FALSE)
	{
		return FALSE;
	}

	while (success == FALSE && try < session->retries)
	{	
		try++;
		if(UdpSend(session->sock, rqs, rqsLen) == FALSE)
		{
			return FALSE;
		}
		while (success == FALSE) 
		{
			if(UdpRecv(session->sock,&rsp,&rspLen, 
					session->timeout) == FALSE)
			{
				break;
			}
			if (SnmpDec(rsp, rspLen, &pdu, commRsp, sizeof(commRsp),
				&commLenRsp, list, listLen, &listLenRsp))
			{
				if (pdu.Request.Id == id)
				{
					*status = pdu.Request.ErrorStatus;
					*index = pdu.Request.ErrorIndex;
					success = TRUE;
				}
			}
		}
	}
	if (success == FALSE)
		strcpy(snmpError,"timeout on SNMP operation");
	return success;
}


int snmpGet(session, vars, numVars)
	snmp	*session;
	snmpvar	*vars;
	int	numVars;
{
        WORD status, index;
	int	len;
	snmpvar	*curVar;

	if (!snmpRequest(session,SNMP_PDU_GET,vars,numVars,&status,&index))
	{
		return(-1);
	}
	if (status != SNMP_NOERROR)
	{
		curVar = vars + index - 1;
                MibID2Name(curVar->Id, (int)(curVar->IdLen), &len,
			curVar->textID);
		sprintf(snmpError,"%s, %s",SNMP_Errors[status],curVar->textID);
		return(-1);
	} 
	return(snmpProcessResult(vars, numVars));
}



int snmpGetNext(session, vars, numVars)
	snmp	*session;
	snmpvar	*vars;
	int	numVars;
{
        WORD status, index;
	int	len;
	snmpvar	*curVar;

	if (!snmpRequest(session,SNMP_PDU_NEXT,vars,numVars,&status,&index))
	{
		return(-1);
	}
	if (status != SNMP_NOERROR)
	{
		curVar = vars + index - 1;
                MibID2Name(curVar->Id, (int)(curVar->IdLen), &len,
			curVar->textID);
		sprintf(snmpError,"%s, %s",SNMP_Errors[status],curVar->textID);
		return(-1);
	} 
	return(snmpProcessResult(vars, numVars));
}
